#include "Individual.h" 

void Individual::evaluateCompleteCost()
{
	myCostSol = CostSol();
	for (int r = 0; r < params->nbVehicles; r++)
	{
		if (!chromR[r].empty())
		{
			double distance = params->timeCost[0][chromR[r][0]];
			double load = params->cli[chromR[r][0]].demand;
			double service = params->cli[chromR[r][0]].serviceDuration;
			predecessors[chromR[r][0]] = 0;
			for (int i = 1; i < (int)chromR[r].size(); i++)
			{
				distance += params->timeCost[chromR[r][i-1]][chromR[r][i]];
				load += params->cli[chromR[r][i]].demand;
				service += params->cli[chromR[r][i]].serviceDuration;
				predecessors[chromR[r][i]] = chromR[r][i-1];
				successors[chromR[r][i-1]] = chromR[r][i];
			}
			successors[chromR[r][chromR[r].size()-1]] = 0;
			distance += params->timeCost[chromR[r][chromR[r].size()-1]][0];
			myCostSol.distance += distance;
			myCostSol.nbRoutes++;
			if (load > params->vehicleCapacity) myCostSol.capacityViol += load - params->vehicleCapacity;
			if (distance + service > params->durationLimit) myCostSol.lengthViol += distance + service - params->durationLimit;
		}
	}

	myCostSol.penalizedCost = myCostSol.distance + myCostSol.capacityViol*params->penalityCapa + myCostSol.lengthViol*params->penalityLength;
	isFeasible = (myCostSol.capacityViol < MY_EPSILON && myCostSol.lengthViol < MY_EPSILON);
}

double Individual::distance(Individual * indiv1, Individual * indiv2)
{
	int note = 0;
	for (int j = 1; j <= indiv1->params->nbClients; j++)
	{
		if (indiv1->successors[j] != indiv2->successors[j] && indiv1->successors[j] != indiv2->predecessors[j]) note++;
		if (indiv1->predecessors[j] == 0 && indiv2->predecessors[j] != 0 && indiv2->successors[j] != 0) note++;
	}
	return ((double)note/ (double)indiv1->params->nbClients);
}

void Individual::addProximity(Individual * indiv)
{
	ProxData data(indiv, Individual::distance(this, indiv));
	if (closestIndividuals.empty()) closestIndividuals.push_back(data);
	else
	{
		auto it = closestIndividuals.begin();
		while ( it != closestIndividuals.end() && it->dist < data.dist) ++it ;
		closestIndividuals.insert(it,data);
	}
}

void Individual::removeProximity(Individual * indiv)
{
	for (auto it = closestIndividuals.begin(); it != closestIndividuals.end(); ++it)
	{
		if (it->indiv == indiv)
		{
			closestIndividuals.erase(it);
			break;
		}
	}
}

double Individual::averageDistanceClosest(int n) 
{
	double result = 0 ;
	double compte = 0 ;
	auto it = closestIndividuals.begin();
	for (int i=0 ; i < n && it!= closestIndividuals.end(); i++)
	{
		result += it->dist ;
		compte += 1.0 ;
		++it ;
	}
	return result/compte ;
}

void Individual::copy(Individual * destination, Individual * source)
{
	destination->chromT = source->chromT;
	destination->chromR = source->chromR;
	destination->successors = source->successors;
	destination->predecessors = source->predecessors;
	destination->myCostSol = source->myCostSol;
	destination->biasedFitness = source->biasedFitness;
	destination->isFeasible = source->isFeasible;
}

Individual::Individual(Params * params, bool createAllStructures) : params(params)
{
	if (createAllStructures)
	{
		successors = std::vector <int>(params->nbClients + 1);
		predecessors = std::vector <int>(params->nbClients + 1);
		chromR = std::vector < std::vector <int> >(params->nbVehicles);
		chromT = std::vector <int>(params->nbClients);
		for (int i = 0; i < params->nbClients; i++) chromT[i] = i + 1;
		std::random_shuffle(chromT.begin(), chromT.end());
	}
}