#ifndef INDIVIDUAL_H
#define INDIVIDUAL_H

#include "Params.h"

/**
 * Author: Thibaut Vidal
 * 
 * The chromossome (a solution) of HGSADC is defined in this class.
 *
 */

class Individual;

struct CostSol
{
	double penalizedCost;		// Penalized cost of the individual
	int nbRoutes;				// Number of routes
	double distance;			// Distance
	double capacityViol;		// Violation of capacity constraints
	double lengthViol;			// Violation of duration constraints
	CostSol() { penalizedCost = 0.; nbRoutes = 0; distance = 0.; capacityViol = 0.; lengthViol = 0.; }
};

struct ProxData
{
	Individual * indiv;			// Individual references in the proximity list
	double dist;				// Distance to this individual
	ProxData(Individual * indiv, double dist): indiv(indiv), dist(dist) {};
};

class Individual
{
public:

  Params * params ;									// Problem parameters
  CostSol myCostSol;								// Solution cost parameters
  bool isFeasible;									// Feasibility status of the individual
  std::vector < int > chromT ;						// Giant tour representing the individual
  std::vector < std::vector <int> > chromR ;		// For each vehicle, the associated sequence of deliveries (complete solution)
  std::vector < int > successors ;					// For each node, the successor in the solution (can be the depot 0)
  std::vector < int > predecessors ;				// For each node, the predecessor in the solution (can be the depot 0)
  double biasedFitness ;							// Biased Fitness of the solution
  std::list < ProxData > closestIndividuals ;		// List of other individuals in the population, ranked by proximity (used by the diversity management procedure)
  
  // Measuring cost of a solution from the information of chromR
  void evaluateCompleteCost();

  // Adding an individual in the structure of proximity
  void addProximity(Individual * indiv) ;

  // Removing an individual in the structure of proximity
  void removeProximity(Individual * indiv);

  // Returns the average distance with the n closest elements
  double averageDistanceClosest(int n) ;

  // Distance measure between individuals
  static double distance(Individual * indiv1, Individual * indiv2);

  // Copy of an Individual into another
  static void copy(Individual * destination, Individual * source);

  // Constructor
  Individual(Params * params, bool createAllStructures);
};
#endif
