#include "Genetic.h"
#include "commandline.h"
#include "LocalSearch.h"
#include "Split.h"
#include "Mining.h"
using namespace std;

int main(int argc, char *argv[])
{
	try
	{
		commandline c(argc, argv);
		if (c.is_valid())
		{
			// Initialization of the problem data from the commandline
			Params params(c.get_path_to_instance(), c.get_path_to_solution(), c.get_type(), c.get_nbVeh(), c.get_seed(), c.get_settingPILS());

			// Maximum time for the program
			// If not specified, setting 240seconds of CPU time per 100 customers (To compare with Arnold et al. in PILS works)
			clock_t nb_ticks_allowed;
			if (c.get_cpu_time() != 0) nb_ticks_allowed = c.get_cpu_time() * CLOCKS_PER_SEC;
			else nb_ticks_allowed = (int)(2.40 * (double)params.nbClients) * CLOCKS_PER_SEC; 

			// Creating the Split algorithm
			Split split(&params);

			// Creating the structure to mine frequent sequences
			Mining mining(&params);

			// Creating a local search algorithm
			LocalSearch localSearch(&params,&mining);

			// Some traces
			std::cout << "----- INSTANCE LOADED: " << params.nbClients << " CUSTOMERS AND " << params.nbVehicles << " VEHICLES -----" << std::endl;
			std::cout << "----- MAX COMPUTATIONAL TIME ALLOCATED: " << nb_ticks_allowed/CLOCKS_PER_SEC << " seconds -----" << std::endl;
			std::cout << "----- PILS: " << params.settingPILS << " -----" << std::endl;
			std::cout << "----- LMAX: " << params.maxSeqMining << " -----" << std::endl;
			std::cout << "----- PHI_FREQ: " << params.factorPatternMining << " -----" << std::endl;
			std::cout << "----- PHI_SIZE: " << params.factorPatternTesting << " -----" << std::endl;
			std::cout << "----- P_EX: " << 1.0/(double)params.samplingRatioMining << " -----" << std::endl;
	
			// Creating an initial population
			std::cout << "----- BUILDING INITIAL POPULATION -----" << std::endl;		
			Population population(&params, &split, &localSearch, &mining);

			// Creating and running the genetic algorithm
			std::cout << "----- RUNNING GENETIC ALGORITHM -----"<< std::endl;
			
			Genetic solver(&params, &split, &population, &localSearch, &mining, nb_ticks_allowed);
			solver.run(100000000, 100000000); // For PILS calibration tests, the only termination criterion is based on time

			// Exporting the best solution
			population.exportBestSolution(params.pathToSolution);
			std::cout << "----- END OF THE ALGORITHM -----" << std::endl;
		}
	}
	catch (const string& e) { std::cout << "EXCEPTION | " << e << std::endl; }
	catch (const std::exception& e) { std::cout << "EXCEPTION | " << e.what() << std::endl; }
	return 0;
}
